use crate::{config::GiteaConfig, revision::Revision};
use reqwest::{
    header::{HeaderMap, HeaderValue},
    Client, StatusCode,
};

#[derive(Clone, Debug, serde::Serialize)]
struct GiteaTag {
    tag_name: String,
}

#[derive(Clone, Debug, serde::Deserialize)]
struct GiteaTagResponse {
    name: String,
}

#[derive(Debug)]
enum TagError {
    Status(StatusCode),
}

impl std::fmt::Display for TagError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::Status(code) => {
                write!(f, "Failed to create tag, invalid response status: {}", code)
            }
        }
    }
}

impl std::error::Error for TagError {}

#[tracing::instrument(skip_all)]
pub(crate) fn build_client(gitea_token: &str) -> color_eyre::eyre::Result<Client> {
    let mut headers = HeaderMap::new();
    headers.insert(
        "Authorization",
        HeaderValue::from_str(&format!("token {}", gitea_token))?,
    );

    let client = reqwest::Client::builder()
        .default_headers(headers)
        .user_agent("release-checker (+https://git.asonix.dog/asonix/release-checker)")
        .build()?;

    Ok(client)
}

#[tracing::instrument(skip(client))]
pub(crate) async fn get_previous_revision(
    client: &Client,
    config: &GiteaConfig,
) -> color_eyre::eyre::Result<Revision> {
    let tags_url = format!(
        "https://{}/api/v1/repos/{}/{}/tags?page=1&limit=4",
        config.domain, config.owner, config.repo
    );
    let tags_response = client.get(tags_url).send().await?;

    let tags: Vec<GiteaTagResponse> = tags_response.json().await?;

    let revision = if let Some(tag) = tags.get(0) {
        tracing::info!("tag: {}", tag.name);
        tag.name.parse()?
    } else {
        Revision::default()
    };

    Ok(revision)
}

#[tracing::instrument(skip(client))]
pub(crate) async fn create_tag(
    client: &Client,
    config: &GiteaConfig,
    revision: &Revision,
) -> color_eyre::eyre::Result<()> {
    let tag_url = format!(
        "https://{}/api/v1/repos/{}/{}/tags",
        config.domain, config.owner, config.repo
    );

    let response = client
        .post(tag_url)
        .json(&GiteaTag {
            tag_name: revision.to_string(),
        })
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(TagError::Status(response.status()).into());
    }

    Ok(())
}
